#!/usr/bin/python

 ###############################################################################
 #                   Simple Conficker Scanner v2
 #                      (single host scanner)
 #
 #
 # Copyright (C) 2009  Felix Leder & Tillmann Werner
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 3
 # of the License, or (at your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 # 
 # 
 #             contact {leder,werner}@cs.uni-bonn.de
 #
 #
 #  Thanks to Dan Kaminsky and Georg Wichersky for their contributions.
 #
 ###############################################################################


import struct
import sys

from struct import pack
try:
	from impacket import smb
	from impacket import uuid
	from impacket.dcerpc import dcerpc
	from impacket.dcerpc import transport
except:
	print "[ERROR]    Impacket package not found."
	sys.exit(1)


class Request:
	def __init__(self, path):
		self.path = path

		self.stub = '\x01\0\0\0\x02\0\0\0\0\0\0\0\x02\0\0\0\0\0\0\0'
		self.stub += pack('III', len(self.path)/2, 0, len(self.path)/2)
		self.stub += self.path
		self.stub += '\x02\0\0\0\x02\0\0\0\0\0\0\0\x02\0\0\0\0\0\0\0\x01\0\0\0\x01\0\0\0'

	def run(self):
		self.dce.call(0x1f, self.stub)


class Scan_BC(Request):
	def __init__(self):
		path = '\x5c\0\x2e\0\x2e\0\x5c\0\0\0\0\0'
		Request.__init__(self, path)


class Scan_D(Request):
	def __init__(self):
		path = '\x5c\0\0\0'
		Request.__init__(self, path)


def test_infection(ip):
	retval = 0

	# INIT
	try:
		t = transport.DCERPCTransportFactory('ncacn_np:%s[\\pipe\\browser]' % ip)
		t.set_dport(445)
		t.connect()
	except:
		print "[UNKNOWN]  %s: No response from port 445/tcp." % ip
		return 0	

	
	# NetpwPathCanonicalize
	try:
		dce = t.DCERPC_class(t)
		dce.bind(uuid.uuidtup_to_bin(('4b324fc8-1670-01d3-1278-5a47bf6ee188', '3.0')))		

		scanner = Scan_BC()
		scanner.dce = dce
		scanner.run()
		response_BC = dce.recv()

		scanner = Scan_D()
		scanner.dce = dce
		scanner.run()
		response_D = dce.recv()

	except:
		print "[UNKNOWN]  %s: Unable to run NetpwPathCanonicalize." % ip
		return 0
	

	if len(response_BC) >=4 and len(response_BC) <= 8:
		result = struct.unpack('I', response_BC[:4])
		print "[ERROR]    %s:  Malformed scan, response code 0x%08x." % (ip, result[0])
		retval = -2
	elif (len(response_BC)>=16):		
		result = struct.unpack('IIII', response_BC[:16])

                if result[1]==0x5c450000 and result[3]==0x00000057:
			print "[INFECTED] %s: %s [%s]:  Seems to be infected by Conficker B or C." \
				% (ip, t.get_smb_server().get_server_os(), t.get_smb_server().get_server_lanman())
                        retval = 1
                elif result[1] == 0 and result[3] != 123:
			print "[WARNING]  %s: %s [%s]:  Unknown dcerpc return value 0x%08x." \
				% (ip, t.get_smb_server().get_server_os(), t.get_smb_server().get_server_lanman(), result[3])
			retval = -1
                elif result[1] != 0 and result[1] != 0x5c450000:
			print "[WARNING]  %s: %s [%s]:  Unknown error code 0x%08x." \
				% (ip, t.get_smb_server().get_server_os(), t.get_smb_server().get_server_lanman(), result[1])
			retval = -1


	if retval!=1 and retval!=-2:
		if len(response_D) >=4 and len(response_D) <= 8:
			result = struct.unpack('I', response_BC[:4])
			print "[ERROR]    %s:  Malformed scan, response code 0x%08x." % (ip, result[0])
			retval = -2
		elif (len(response_D)>=16):		
			result = struct.unpack('IIII', response_D[:16])

			if result[1]==0x5c45005c and result[3]==0x00000000:
				print "[INFECTED] %s: %s [%s]:  Seems to be infected by Conficker D!" \
					% (ip, t.get_smb_server().get_server_os(), t.get_smb_server().get_server_lanman())
				retval = 1
			elif result[1] == 0 and result[3] != 123:
				print "[WARNING]  %s: %s [%s]:  Unknown dcerpc return value 0x%08x." \
					% (ip, t.get_smb_server().get_server_os(), t.get_smb_server().get_server_lanman(), result[3])
				retval = -1
			elif result[1] != 0 and result[1] != 0x5c450000:
				print "[WARNING]  %s: %s [%s]:  Unknown error code: 0x%08x" \
					% (ip, t.get_smb_server().get_server_os(), t.get_smb_server().get_server_lanman(), result[1])
				retval = -1

			
	if retval == 0:
		print "[CLEAN]    %s: %s [%s]:  Seems to be clean." \
			% (ip, t.get_smb_server().get_server_os(), t.get_smb_server().get_server_lanman())


	dce.disconnect()

	return retval


def banner():
	print "\n  Simple Conficker Scanner v2 -- (C) Felix Leder, Tillmann Werner 2009\n"


if __name__ == '__main__':
	banner()

	if len(sys.argv) != 2:
		print 'Usage: %s <host>' % sys.argv[0]
		sys.exit(0)

	test_infection(sys.argv[1])
	print
